"""
Defines the possibilities how to show the syntax.
"""

from enum import Enum, auto


class ShowFrontendSyntaxEnum(Enum):
    """
    Defines the possibilities how to show the syntax.
    """

    NONE = auto()
    SHOW = auto()
    FULL = auto()
